/***************************************************************************
                          kcdiroperator.cpp  -  description
                             -------------------
    begin                : Fri Dec 14 2001
    copyright            : (C) 2001 by Christian Fricke & Ren Mrten
    email                : cfricke@kcommander.org
 ***************************************************************************/


#include "kcdiroperator.h"
#include <qsettings.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <stdlib.h>
#include <stdio.h>

kcDirOperator::kcDirOperator()
{
//  setName("DirOperator");
  inZipFile = false;
  zipName = "";

}
kcDirOperator::~kcDirOperator(){
}
void kcDirOperator::setname(QString name)
{
  QString a;
  a.sprintf("%u",uint(&name));
  setName(QString(name) + a);    //Speicherbereich des Namens sollte fuer jeden KCommander der laeuft anders sein
}

void kcDirOperator::reRead()
{
  readDir(directory);
}
void kcDirOperator::readDir(QString dir)
{
  // Als allererstes immer ein Slash anhngen, wenn es sich um ein Dir handelt
  if(dir.right(1) != "/")
    dir += "/";
  directory = dir;

//**************************************************************************************
// ZIP ROUTINEN
//**************************************************************************************

  QFileInfo inf(dir);

  // Endung gz, tgz oder zip?
  if(! inZipFile)
  {                          // sonst macht er beim Hochgehen immer ein Entpacken ... hat Rene Vergessen !!
                             // kann Rene nicht vergessen haben, weil das beim tgz auch nicht stand...habs nur 1:1 bernommen
    if( dir.right(3) == "gz/" ||
        dir.right(4) == "tgz/" ||
        dir.right(4) == "rar/" ||
        dir.right(4) == "ace/" ||
        dir.right(4) == "zip/" )
    {
      inZipFile = true;
      zipName = dir;
      readLocalZipfile(inf.fileName());
      emit dirReady();
      return;
    }
  }
  // isser im tempdir vonnem zip oder tgz file?
  if( dir.contains(".tar.gz") >= 1 ||
      dir.contains(".tgz") >= 1 ||
      dir.contains(".rar") >= 1 ||
      dir.contains(".ace") >= 1 ||
      dir.contains(".zip") >= 1)
  {
    inZipFile = true;
    continueReadLocalZipfile(dir);
    return;
  }

  if(inZipFile && ! directory.contains("/tmp/kcommander/") >= 1)
  {
    removeTmpDir();
    inZipFile = false;
  }
//****************************************************************************************

  // Nun splitten wir auf local, ftp oder smb
  if(dir.left(2) == "//") // SAMBA!!
  {
    readDirSmb();
  }
  else if(dir.left(1) == "/" || dir.left(5) == "file:")
  {
    readDirLocal();
  }
  else if(dir.left(4) == "ftp:")
  {
    readDirFtp();
  }
  else
  {
    readDirLocal();
    return;
  }
  //
  emit dirReady();

}
void kcDirOperator::readDirLocal()
{
    QDir d(directory);
    if (!d.exists())
    {
      emit dirNotExist();
      return;
    }

    d.setFilter( QDir::Files |QDir::Dirs | QDir::Hidden);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    while ( (fi = it.current()) != 0 )
    {
        struct_fileinfo *sfi = new struct_fileinfo;
        if(fi->fileName() != ".")
        {
            int p = fi->permission( QFileInfo::ReadUser)  * QFileInfo::ReadUser +
                    fi->permission( QFileInfo::WriteUser) * QFileInfo::WriteUser +
                    fi->permission( QFileInfo::ExeUser)   * QFileInfo::ExeUser +
                    fi->permission( QFileInfo::ReadGroup) * QFileInfo::ReadGroup +
                    fi->permission( QFileInfo::WriteGroup)* QFileInfo::WriteGroup +
                    fi->permission( QFileInfo::ExeGroup)  * QFileInfo::ExeGroup +
                    fi->permission( QFileInfo::ReadOther) * QFileInfo::ReadOther +
                    fi->permission( QFileInfo::WriteOther)* QFileInfo::WriteOther +
                    fi->permission( QFileInfo::ExeOther)  * QFileInfo::ExeOther;
            sfi->_fileName = fi->fileName().latin1();
            sfi->_permissions = p;
            sfi->_lastModified = fi->lastModified();
            sfi->_owner = fi->owner();
            sfi->_group = fi->group();
            sfi->_isSymLink = fi->isSymLink();
            sfi->_isFile = fi->isFile();
            sfi->_isDir = fi->isDir();
            sfi->_isReadable = fi->isReadable();
            sfi->_directoryname = directory;
            sfi->_fileSize = fi->size();
            if(fi->fileName() != "..")
              emit gotFile(sfi);
            else
              if(directory != "/")
                emit gotFile(sfi);
        }
        ++it;
    }
    qApp->processEvents();
}
void kcDirOperator::readDirFtp()
{
}
void kcDirOperator::readDirSmb()
{
  if(!sambaWorking())
    return;

  if(directory == "//") // auf Server (konfigurierbar!!) alle Rechner lesen
  // + Gesamtes Netzwerk
  {
  }
  else if(directory == "//@/") // Listet Workgroups
  {
    // bsp:  //@/
  }
  else if(directory.left(3) == "//@") // Listet alle Rechner einer Workgroup
  {
    // bsp:  //@DREAM/
  }
  else if(directory.contains('/',false) == 3)
  { //  bsp: //rechner@workgroup/ fr Computer in anderem Gruppenverzeichnis
    // oder  //rechner/           im selben Gruppenverzeichnis
  }
  else if(directory.contains('/',false) == 4)
  {
    // bsp:  //rechner@workgroup/freigabename/
    // oder  //rechner/freigabename/
  }
}
bool kcDirOperator::sambaWorking()
{
  QSettings settings;
  // server fr Samba Netsharing
  QString server = settings.readEntry("/KCommander/SMB/server","localhost");

  proc = new QProcess( this );

  proc->addArgument( "smbclient" );
  proc->addArgument( "-N" );
  proc->addArgument( "-L" );
  proc->addArgument( server );


  connect( proc, SIGNAL(readyReadStdout()),  this, SLOT(readFromStdout()) );
  connect( proc, SIGNAL(readyReadStderr()),  this, SLOT(readFromStderr()) );

  if ( !proc->start() )
  {
    delete proc;
    proc = 0L;
    return false;
  }
  return true;
}
void kcDirOperator::readFromStdout()
{
  while(proc->canReadLineStdout())
  {
    QString s = proc->readLineStdout();
  }
}
void kcDirOperator::readFromStderr()
{
}

void  kcDirOperator::readLocalZipfile(QString fileName)
{
      // first check if we can create a temp
      QDir tmp("/tmp/kcommander");
      if(! tmp.exists())
         system(QString("mkdir /tmp/kcommander >/dev/null 2>&1"));
      if(! tmp.exists())
      {
        QMessageBox::information(0, "System Error","ERROR: can not create temporary directory!",1);
        emit dirNotExist();
        return;
      }

      QString extractCommand;

      inZipFile = true;
      zipName = directory + fileName;
      zipName = zipName.left(zipName.length()-1);
      QString realZip = "\"" + zipName + "\"";

      system(QString("mkdir /tmp/kcommander/") + name() + " >/dev/null 2>&1");        // Darf niemals zu sehen sein !!

      system(QString("chmod 700 /tmp/kcommander/")+ name() + " >/dev/null 2>&1"); // Sicherheitshinweis von FRANK

      if(zipName.right(4) == ".zip")
        extractCommand = (QString("unzip ") + realZip + " -d /tmp/kcommander/"+name());
      else
      if(zipName.right(4) == ".rar")
      {
        extractCommand = (QString("unrar x ") + realZip + " /tmp/kcommander/"+name());
      }
      else
      if(zipName.right(4) == ".ace")
      {
        extractCommand = (QString("unace x ") + realZip + " /tmp/kcommander/"+name());
      }
      else
      {
        extractCommand = (QString("tar xzvf ") + realZip + " -C /tmp/kcommander/"+name());
      }

      extractCommand = extractCommand += "/ >/dev/null 2>&1";
      system(extractCommand);
      directory = QString("/tmp/kcommander/")+name() + "/";
      readDirLocal();
      emit dirReady();

}

void  kcDirOperator::continueReadLocalZipfile(QString dirName)
{
    // ab Tar-gz-fileName Restpfad nehmen und auf /tmp/kcommander/ setzen
    QString newTmpDir;
    if(dirName.find(".zip/") != -1)
      newTmpDir = dirName.right(dirName.length() - dirName.findRev(".zip/",-2)-5);
    else
    if(dirName.find(".rar/") != -1)
      newTmpDir = dirName.right(dirName.length() - dirName.findRev(".rar/",-2)-5);
    else
    if(dirName.find(".ace/") != -1)
      newTmpDir = dirName.right(dirName.length() - dirName.findRev(".ace/",-2)-5);
    else
      newTmpDir = dirName.right(dirName.length() - dirName.findRev("gz/",-2)-3);
    directory = QString("/tmp/kcommander/")+name()+"/" + newTmpDir;
    readDirLocal();
    emit dirReady();
}

void  kcDirOperator::removeTmpDir()
{
//    debug("Das issa");
    system(QString("rm -r /tmp/kcommander/")+ name() + " >/dev/null 2>&1");
}
