/***************************************************************************
                          kciconview.h  -  description
                             -------------------
    begin                : Wed Feb 6 2002
    copyright            : (C) 2002 by Christian Fricke & Ren Mrten
    email                : cfricke@kcommander.org
 ***************************************************************************/


#ifndef KCICONVIEW_H
#define KCICONVIEW_H

#include <qiconview.h>

#include <qsettings.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qdragobject.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qheader.h>
#include <qfileinfo.h>
#include <stdlib.h>

#include "kcdiroperator.h"
#include "kcfileiconprovider.h"
#include "kcview.h"
#include "kcmailus.h"
#include "dlgcopy.h"

class kcIconView;

/**
  *@author Christian Fricke & Ren Mrten
  */

class kcIconViewItem : public QIconViewItem{
//  Q_OBJECT
public:
	kcIconViewItem(kcIconView *parent);
  kcIconViewItem(const struct_fileinfo *fi, kcIconView *parent, QIconViewItem *after= NULL,const QString &text=QString::null);
  ~kcIconViewItem();
  virtual int compare( QIconViewItem *i ) const;
  bool          isFile() const;
  bool          isDir() const;
  bool          isSymLink() const;
  bool          isReadable() const;
  QDateTime     lastModified() const;
  int           permissions() const;
  uint          fileSize() const;
  QString         fullPath()
                    { return _fullPath;};

  void          paintItem( QPainter *p, const QColorGroup &cg);
  virtual       QPixmap *pixmap () const;
  virtual       QString text () const;

  bool        iconFlag;
private:
  QTimer  *iconTimer;
  QString         _fullPath;
  kcIconView *iconView;
  struct_fileinfo  *finfo;
//private slots:
//  void        findItem();
};

/**
  *@author Christian Fricke & Ren Mrten
  */

//class kcIconView;


class kcIconView : public QIconView , public kcView  {
  Q_OBJECT
public:
	kcIconView(QWidget *parent=0, const char *name=0);
	~kcIconView();
  void          setDir(QString path);
  QString       dirname();
  QString       realDirname(){return dirOperator->dirname();};
	void 			reRead();
  void          sendKey(QKeyEvent *);
  QColor  getBackgroundColor();
  QColor  getHighlightColor();

  QString       fullPath(QIconViewItem* item);

  /** If the user had changed the settings, using optionsdialog, we have to react to.
  * Function is called by kcommander3view.cpp
  */
  void              settingsChanged();
//  kcFileIconProvider *fip;

signals:
//  // This Signal is committed if we want show any info to the user
  void          pathButtonTextChanged(QString );
  void          focusOut();
  void          focusIn();
  void          selectionChanged();
//  void          popupEvent(int menuEntry);
  void          activateCommandBox(QString);
  void          dirChanged(QString &);
  void          changeViewer(QString);
  void          changePropertiesForFile(QFileInfo);
  void          makeSymlinkToFile(QFileInfo);

public slots:
  void          slotCTRL_PlusPressed();
  void          slotCTRL_MinusPressed();
  void          slotMinusPressed();
  void          slotPlusPressed();
  void          slotInvertSelection();

// Kontextmenu SLOTs /////////////////////////
  void          makeViewChange();
  void          makePropertiesForFileChange();
  void          makeSymlink();
  void            sendFileAsMail();
///////////////////////////////////////////////////////////
private slots:
  void          contextMenuShow();
  void          keyPressEvent(QKeyEvent *);
  void          gotFileList(QPtrList<struct_fileinfo> *);
  void          gotFile(const struct_fileinfo *);
  void          dirReady();
  void          dirNotExist();
  // ItemSignals
  void          slotFolderSelected(QIconViewItem *item);
////  void          doubleClicked(QListViewItem *item);

  void          contentsDragEnterEvent( QDragEnterEvent *);
  void          contentsDragMoveEvent( QDragMoveEvent *);
  void          contentsDragLeaveEvent( QDragLeaveEvent *);
  void          contentsDropEvent( QDropEvent *);

  void          contentsMousePressEvent( QMouseEvent *);
  void          contentsMouseReleaseEvent( QMouseEvent *);
  void          contentsMouseMoveEvent( QMouseEvent *);
  void          contentsContextMenuEvent ( QContextMenuEvent *);

// Kontextmenu SLOTs /////////////////////////
  void          getColorDialog();
  void          renameFile() ;
  void          itemWithNewName(QIconViewItem *, const QString &);
  void          setShowDotFiles();
///////////////////////////////////////////////////////////

  void          focusInEvent(QFocusEvent *e);
  void          focusOutEvent(QFocusEvent *e);

private:
  kcIconViewItem  *dragItem;
  int           mouseX, mouseY; // POS where mouse was pressed
  QPopupMenu    *popupMenu;
  QString       directory;
  QString       oldDirectory;
  QString       n; //name !!
  kcDirOperator *dirOperator;
  QTimer        *contextTimer;
  void          refreshInfo();

  int           actItemWidth;

  bool         startProgram(QFileInfo);
  QString   getExpression();
  void         setSelection(QString exp, bool set);

// Colors
  QColor bg;
  QColor hl;

// Drag'n Drop
  QIconViewItem *firstSelectedItem;
  QFileInfo   renFileInfo;
  bool          mark;
  bool          dirsOnly;
  QIconViewItem *oldCurrent;
  QIconViewItem *dropItem;
  QIconViewItem *RMBpressedOn;
  QPoint        presspos;
  bool          mousePressed;
  int           button;


  bool      inZipFile;

// config
  bool          showDotFiles;
  bool          countReady;

#define autoopenTime 10
};

#endif
