/***************************************************************************
                          kclistview.h  -  description
                             -------------------
    begin                : Thu Dec 13 2001
    copyright            : (C) 2001 by Christian Fricke & Ren Mrten
    email                : cfricke@kcommander.org
 ***************************************************************************/


#ifndef KCLISTVIEW_H
#define KCLISTVIEW_H

#include <qsettings.h>
#include <qwidget.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qdragobject.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qheader.h>
#include <qfileinfo.h>
#include <qtranslator.h>
#include <qaction.h>
#include <qaccel.h>
#include <qcolordialog.h>
#include <qpainter.h>

#include "kcdiroperator.h"
#include "kcfileiconprovider.h"
#include "kcview.h"
#include "kcsimplebutton.h"
#include "kcmailus.h"
#include "dlgcopy.h"

/**
  *@author Christian Fricke & Ren Mrten
  */
class kcListView;

class kcListViewItem : public QListViewItem {
public:
	kcListViewItem(kcListView *parent);
  kcListViewItem(   const struct_fileinfo *fi,
                    kcListView *parent,QString l1, QString l2=QString::null,
                    QString l3=QString::null,QString l4=QString::null,QString l5=QString::null,
                    QString l6=QString::null,QString l7=QString::null,QString l8=QString::null);
  ~kcListViewItem();
  int             compare( QListViewItem * i, int col, bool ascending ) const;
  bool            isFile() const;
  bool            isDir() const;
  bool            isSymLink() const;
  bool            isReadable() const;
  QDateTime       lastModified() const;
  int             permissions() const;
  uint            fileSize() const;
  QString         text(int column) const;
  QString         fullPath()
                    { return _fullPath;};
  void            paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int alignment );

//  virtual void paintFocus( QPainter *, const QColorGroup &,
//                                const QRect &);

  bool            iconFlag;
  struct_fileinfo *getFInfo(){return finfo;};
private:

  kcListView      *listView;
  struct_fileinfo *finfo;
  QTimer          *iconTimer;
  QString         _fullPath;
};

#define autoopenTime 10

/**
  *@author Christian Fricke & Ren Mrten
  */
class kcListView : public QListView, public kcView  {
  Q_OBJECT
public:
	kcListView(QWidget *parent=0, const char *name=0);
	~kcListView();
  void              setDir(QString path);
  QString         dirname();
  QString         realDirname(){return dirOperator->dirname();};

	void 			   reRead();
  void              sendKey(QKeyEvent *);
  QColor          getBackgroundColor();
  QColor          getHighlightColor();
  bool              itemLightColor;
  QString         fullPath(QListViewItem* item);

  /** If the user had changed the settings, using optionsdialog, we have to react to.
  * Function is called by kcommander3view.cpp
  */
  void              settingsChanged();
signals:
  // This Signal is committed if we want show any info to the user
  void              pathButtonTextChanged(QString );
  void              focusOut();
  void              focusIn();
  void              selectionChanged();
  void              popupEvent(int menuEntry);
  void              activateCommandBox(QString);
  void              dirChanged(QString &);
  void              changeViewer(QString);
  void              changePropertiesForFile(QFileInfo);
  void              makeSymlinkToFile(QFileInfo);

public slots:
  void              slotCTRL_PlusPressed();
  void              slotCTRL_MinusPressed();
  void              slotMinusPressed();
  void              slotPlusPressed();
  void              slotInvertSelection();

// Kontextmenu SLOTs /////////////////////////
  void          makeViewChange();
  void          makePropertiesForFileChange();
  void          makeSymlink();
  void            sendFileAsMail();
///////////////////////////////////////////////////////////
private slots:
  void            readFromStdout();
  void            contextMenuShow();
  void            keyPressEvent(QKeyEvent *);
//  void          keyReleaseEvent(QKeyEvent *);
  void            gotFileList(QPtrList<struct_fileinfo> *);
  void            gotFile(const struct_fileinfo *);
  void            dirReady();
  void            dirNotExist();
  // ItemSignals
  void            slotFolderSelected(QListViewItem *item);
//  void          doubleClicked(QListViewItem *item);

  void            contentsDragEnterEvent( QDragEnterEvent *);
  void            contentsDragMoveEvent( QDragMoveEvent *);
  void            contentsDragLeaveEvent( QDragLeaveEvent *);
  void            contentsDropEvent( QDropEvent *);

  void            contentsMousePressEvent( QMouseEvent* e );
  void            contentsMouseReleaseEvent( QMouseEvent * );
  void            contentsMouseMoveEvent( QMouseEvent* e );
  void            contentsContextMenuEvent ( QContextMenuEvent * e );

// Kontextmenu SLOTs /////////////////////////
  void            getColorDialog();
  void            renameFile() ;
  void            itemWithNewName(QListViewItem *,int, const QString &);
  void            setShowDotFiles();
///////////////////////////////////////////////////////////

  void            focusInEvent(QFocusEvent *e);
  void            focusOutEvent(QFocusEvent *e);


private:
  kcListViewItem  *dragItem;

  QProcess        *proc;
  bool            countReady;
  int             mouseX, mouseY; // POS where mouse was pressed
  QPopupMenu      *popupMenu;
  QString         directory;
  QString         oldDirectory;
  QString         n; //name !!
  kcDirOperator   *dirOperator;
  QTimer          *contextTimer;
  void            refreshInfo();

  bool            startProgram(QFileInfo);

// Colors
  QColor          bg;
  QColor          hl;

// Drag'n Drop

  QListViewItem   *firstSelectedItem;
  QFileInfo       renFileInfo;
  bool            mark;
  bool            dirsOnly;
  QListViewItem   *oldCurrent;
  QListViewItem   *dropItem;
  QListViewItem   *RMBpressedOn;
  QPoint          presspos;
  bool            mousePressed;
  int             button;
  QString         getExpression();
  void            setSelection(QString exp, bool set);
  bool            inZipFile;
// config
  bool            showDotFiles;

  void            startReadIcons();
  void            stopReadIcons();
  kcListViewItem  *item;
  int             countVisited();
};

#endif
