kcListViewItem::kcListViewItem(kcListView *parent) : QListViewItem(parent)
{
}
kcListViewItem::kcListViewItem(
                const struct_fileinfo *fi,
                kcListView *parent,
                QString l1,QString l2,QString l3,QString l4,
                QString l5,QString l6,QString l7,QString l8)
                 : QListViewItem(parent,l1,l2,l3,l4,l5,l6,l7,l8)
{
  iconFlag = false;
  listView = parent;
  finfo = (struct_fileinfo *)fi;
  _fullPath = fi->_directoryname;
  if(_fullPath.right(1) != "/")
    _fullPath += "/";
  _fullPath += l1;
  if(isDir())
  {
    setText(1,"Directory");
    if(isSymLink())
      setPixmap(0,*listView->fip->getIcon("sfolder"));
    else{
      if(QDir::home().absPath() == _fullPath) setPixmap(0,*listView->fip->getIcon("home"));
      else if(fi->_fileName == "..") setPixmap(0,*listView->fip->getIcon("back"));
      else setPixmap(0,*listView->fip->getIcon("folder"));
      }
    iconFlag = true;
  }
  else
  {
    if(isSymLink())
    {
      setPixmap(0,*listView->fip->getIcon("sfile"));
      iconFlag = true;
    }
    else
      setPixmap(0,*listView->fip->getIcon(_fullPath));
  }
  setRenameEnabled(0,true);
}

QString kcListViewItem::text(int column) const
{
  if(QListViewItem::text(0) == "..")
  switch(column)
  {
    case 0:
      return ".."; break;
    case 1:
      return "Directory"; break;
    default:
      return ""; break;
  }

  switch(column)
  {
    case 0:
      return QListViewItem::text(column);
      break;
    case 1:
      if(isDir())
        return QListViewItem::text(column);
      else
        {
          QString s; s.sprintf("%d",fileSize());
          if(s.length()>3)
            s.insert(s.length()-3,".");
          if(s.length()>7)
            s.insert(s.length()-7,".");
          return s;
        }
      break;
    case 2: //date
      {
      QString s = finfo->_lastModified.toString("yyyy-MM-dd");
      return s;
      break;
      }
    case 3: //permissions
      {
      QString s;
      int mode = permissions();
      s.sprintf("%s%s%s",rwx[(mode >> 6) & 0x7],rwx[(mode >> 3) & 0x7],rwx[mode        & 0x7]);
      return s;
      break;
      }
    case 4: //owner
      return finfo->_owner;
      break;
    case 5: //group
      return finfo->_group;
      break;
    default:
      return ("...");
  }
  return "";
}

kcListViewItem::~kcListViewItem(){
}

int kcListViewItem::compare(QListViewItem * i, int col, bool ascending ) const
{
  kcListViewItem *li = (kcListViewItem *)i;
  // TEST

  // ".." muss immer oben sein!
  //*********************************
  if(li->key(0,true) == "..")
    if(ascending)
      return 1;
    else
      return -1;
  if(key(0,true) == "..")
    if(ascending)
      return -1;
    else
      return 1;
  //*********************************

  // Dirs sind immer ber Files
  //*********************************
  if(isFile() && li->isDir())
    if(ascending)
      return 1;
    else
      return -1;
  if(isDir() && li->isFile())
    if(ascending)
      return -1;
    else
      return 1;
  //*********************************

  switch(col)
  {
    case 0:
      return key(col, ascending).localeAwareCompare( li->key(col, ascending));
      break;

    // Nach Size sortieren
    case 1:
      if(isDir() && li->isDir())
        return key(0, ascending).localeAwareCompare( li->key(0, ascending));
      if(li->fileSize() == fileSize())
        return key(0, ascending).localeAwareCompare( li->key(0, ascending));
      if(li->fileSize() < fileSize())
        return 1;
      else
        return -1;
      break;

    // nach Datum sortieren
    case 2:
      if(lastModified() == li->lastModified())
        return key(0, ascending).localeAwareCompare( li->key(0, ascending));
      if(lastModified() < li->lastModified())
        return -1;
      else
        return 1;
      break;

    // diese Spalten werden normal nach Text sortiert

    case 3:     // nach Rechten sortieren
    case 4:     // nach Owner sortieren
    case 5:     // nach Group sortieren
        if(li->text(col) == text(col))
          return li->text(0).compare(text(0));
        if(li->text(col) == text(col))
          return li->text(0).compare(text(0));
      break;
  }
  return key(col, ascending).localeAwareCompare( li->key(col, ascending));
}

bool kcListViewItem::isFile() const
{
  return finfo->_isFile;
}
QDateTime kcListViewItem::lastModified() const
{
  return finfo->_lastModified;
}
bool kcListViewItem::isDir() const
{
  return finfo->_isDir;
}
bool kcListViewItem::isSymLink() const
{
  return finfo->_isSymLink;
}
bool kcListViewItem::isReadable() const
{
  return finfo->_isReadable;
}
uint kcListViewItem::fileSize() const
{
  return finfo->_fileSize;
}
int kcListViewItem::permissions() const
{
  return finfo->_permissions;
}

void kcListViewItem::paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int alignment )
{
    QColorGroup cGroup( cg );
    if(listView->recolor)
    {
      if(column%2 == 1)
      {
        cGroup.setColor( QColorGroup::Base, listView->getBackgroundColor() );      //Base = Hintergrund  und  Text = Textfarbe !!
      }
      else
      {
        cGroup.setColor( QColorGroup::Base, listView->getBackgroundColor().dark(108) );      //Base = Hintergrund  und  Text = Textfarbe !!
      }
    }
    else
    {
      cGroup.setColor( QColorGroup::Base, listView->getBackgroundColor() );      //Base = Hintergrund  und  Text = Textfarbe !!
    }
    cGroup.setColor( QColorGroup::Highlight, listView->getHighlightColor());      //Highlight = Selected

    if  ( isSymLink() )
    {
      QFont f( p->font() );
      f.setItalic(listView->itemItalic);  // configurable
      f.setBold(listView->itemBold);
      p->setFont( f );
    } ;
    QListViewItem::paintCell( p, cGroup, column, width, alignment );
}
