/***************************************************************************
                          kcsimplebutton.cpp  -  description
                             -------------------
    begin                : Tue Dec 11 2001
    copyright            : (C) 2001 by Christian Fricke & Ren Mrten
    email                : cfricke@kcommander.org
 ***************************************************************************/


#include "kcsimplebutton.h"
#include <qpainter.h>
#include <qtooltip.h>
#include <qapplication.h>

kcSimpleButton::kcSimpleButton(QPopupMenu *p, QWidget *parent, const char *name ) : QButton(parent,name)
{
  popup = p;
  kcSimpleButton(parent, name );
  connect(popup, SIGNAL(activated(int)), SLOT(popupActivated(int)));
}
kcSimpleButton::kcSimpleButton(QWidget *parent, const char *name ) : QButton(parent,name)
{
  popupTimer = new QTimer(this);
  connect(popupTimer, SIGNAL(timeout()), SLOT(startPopup()));
  popup = 0;
  mouseover = false;
  mousepressed = false;
  isActive = false;
  text = "";
  setFocusPolicy(NoFocus);
}
kcSimpleButton::~kcSimpleButton(){
}
void kcSimpleButton::setText(QString t)
{
  QString originalText = t;
//  filterPassword();
  // cutting String if it is too long
  int max = width();
  QString tempString(originalText);
  if(fontMetrics().width(tempString) >= max)
  {
    QToolTip::add(this,originalText);
    while(fontMetrics().width("........" + tempString) >= max)
    {
      tempString = tempString.right(tempString.length()-1);
    }
    tempString = "..." + tempString;
  }
  else
  QToolTip::add(this,"");
  text = tempString;


  update();
}
void kcSimpleButton::mousePressEvent(QMouseEvent *e)
{
  pos = QPoint(e->globalPos());
  popupTimer->start(1000,true);
  mousepressed = true;
  update();
  e->ignore();
}
void kcSimpleButton::mouseReleaseEvent(QMouseEvent *e)
{
  popupTimer->stop();
  mousepressed = false;
  update();
  if( mouseover )
    emit(clicked());
  e->ignore() ;
}
void kcSimpleButton::enterEvent(QEvent *e)
{
  mouseover = true;
  update();
}
void kcSimpleButton::leaveEvent(QEvent *e)
{
  mouseover = false;
  update();
}
void kcSimpleButton::paintEvent(QPaintEvent *e)
{
  // nderung:
  // ist Maus nicht berm Button, so wird kein Rand gemalt
  // wenn Maus drber ist, wird Button herausgezogen
  // wenn Maus gedrckt, ist Button auch gedrckt

  QColor c1 = QColor(255,255,255);  // light Boarder
  QColor c2 = QColor(80,80,80);     // dark Boarder
  QColor c3;
  QColor c4(130,130,130);

  QPainter p(this);
  if(isActive)
  {
    p.fillRect(2,2,width()-4,height()-4,QColor(180,180,255));
  }

  if(mouseover)
  {
    if(mousepressed)
    {
      // Farben vertauschen
      c3 = c2; c2 = c1; c1 = c3;
    }
    p.setPen(c1);
    p.drawLine(0,0,width()-1,0);
//    p.drawLine(1,1,width()-2,1);

    p.drawLine(0,0,0,height()-1);
//    p.drawLine(1,1,1,height()-2);
    p.setPen(c2);

    p.drawLine(width()-1,0,width()-1,height()-1);
//    p.drawLine(width()-2,1,width()-2,height()-2);

    p.drawLine(0,height()-1,width()-1,height()-1);
//    p.drawLine(1,height()-2,width()-2,height()-2);
  }
  if(text != "")
  {
    p.setPen(black);        //  ist von Arno noch mal zu bearbeiten
    p.drawText(QRect(0,0,width(),height()), AlignCenter, text);
  }
  if(popup)
  {
    p.setPen(c4);
    int x = width()-12;
    int y = height()-12;
    p.drawLine(x,y,x+6,y);
    p.drawLine(x+1,y+2,x+8,y+2);
    p.drawLine(x+1,y+4,x+8,y+4);
    p.drawLine(x+1,y+6,x+8,y+6);
    p.drawLine(x+1,y+8,x+8,y+8);
    //p.fillRect(3,3,5,5,c4);
  }
}

void kcSimpleButton::setActiveFlag(bool flag)
{
  isActive = flag;
  repaint();
}

void  kcSimpleButton::setPopupMenu(QString s1,QString s2,QString s3,QString s4,QString s5,QString s6,QString s7,QString s8,QString s9,QString s10,QString s11)
{
  if(popup)
    delete popup;
  popup = new QPopupMenu(this);
  if(s1)
    popup->insertItem(s1,1);
  if(s2)
    popup->insertItem(s2,2);
  if(s3)
    popup->insertItem(s3,3);
  if(s4)
    popup->insertItem(s4,4);
  if(s5)
    popup->insertItem(s5,5);
  if(s6)
    popup->insertItem(s6,6);
  if(s7)
    popup->insertItem(s7,7);
  if(s8)
    popup->insertItem(s8,8);
  if(s9)
    popup->insertItem(s9,9);
  if(s10)
    popup->insertItem(s10,10);
  if(s11)
    popup->insertItem(s11,11);
  connect(popup, SIGNAL(activated(int)), SLOT(popupActivated(int)));
}
void  kcSimpleButton::popupActivated(int id)
{
  emit(popupSelected(id));
}
void  kcSimpleButton::startPopup()
{
  if(popup)
    popup->exec(QPoint(pos.x()-10, pos.y()-10));
}
